﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
//using System.Threading.Tasks; // .NET Franework 3.5 では使えないのでコメントアウト
using System.Windows.Forms;
using System.Diagnostics; // ウィンドウプロセス取得のために追加

namespace scClock
{
    public partial class Display : Form
    {
        public Display()
        {
            InitializeComponent();
        }

        /// Timerのメッセージハンドラ
        private void secTimer_Tick(object sender, EventArgs e)
        {
            DisplayTime();
        }

        /// Form生成のメッセージハンドラ
        private void Display_Load(object sender, EventArgs e)
        {
            this.TopMost = true;
            DisplayTime();
        }

        /// 現在時刻の表示
        private void DisplayTime()
        {
            string time=DateTime.Now.ToString("HH:mm:ss");
            this.currentTime.Text = time;
        }

        // Labelのマウス左ボタンダブルクリックのメッセーハンドラ
        private void currentTime_DoubleClick(object sender, EventArgs e)
        {
            //プログラムを終了する
            Dispose();
        }

        ///MouseDown位置を記憶する変数
        private Point mousePoint;

        ///DisplayのMouseDownイベントハンドラ
        ///マウスのボタンが押されたとき
        private void currentTime_MouseDown(object sender, MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                //カーソルをHandに変更
                this.Cursor = Cursors.Hand;
                //位置を記憶する
                mousePoint = new Point(e.X, e.Y);
            }
            else
            {
                //カーソルを元に戻す
                this.Cursor = Cursors.Default;
            }

            if ((e.Button & MouseButtons.Right) == MouseButtons.Right)
            {
                //最小化する
                this.WindowState = FormWindowState.Minimized;
            }
        }

        ///DisplayのMouseMoveイベントハンドラ
        ///マウスが動いたとき
        private void currentTime_MouseMove(object sender, MouseEventArgs e)
        {
            if ((e.Button & MouseButtons.Left) == MouseButtons.Left)
            {
                //カーソルをHandに変更
                this.Cursor = Cursors.Hand;
                //フォームの位置変更
                this.Left += e.X - mousePoint.X;
                this.Top += e.Y - mousePoint.Y;
                this.TopMost = true;
            }
            else
            {
                //カーソルを元に戻す
                this.Cursor = Cursors.Default;
            }
        }

        /// 定期的にアクティブウィンドウを取得する
        private void cursortimer_Tick(object sender, EventArgs e)
        {
            GetActiveWindow();
        }

        /// アクティブなウィンドウを取得する
        private void GetActiveWindow()
        {
            // アクティブウィンドウのハンドルを取得する
            IntPtr hWnd = WinAPI.GetForegroundWindow();
            // アクティブウィンドウの位置を取得
            RECT winRect = new RECT();
            WinAPI.GetWindowRect(hWnd, ref winRect);
            int id; 
            WinAPI.GetWindowThreadProcessId(hWnd, out id);
            string pName = Process.GetProcessById(id).ProcessName;

            // scClock自身がアクティブでない時 || MyClock がアクティブでない時
            if (Form.ActiveForm != this)
            {
                if (pName != "MyClock")
                {
                    // scClockをアクティブウィンドウへ移動する
                    MoveDisplayTo(winRect);
                }
            }
        }

        // キャプションバー上ボタンの寸法取得
        private static int buttonW = SystemInformation.CaptionButtonSize.Width;
        private static int buttonH = SystemInformation.CaptionButtonSize.Height;
        // ウィンドウ枠の幅を取得
        private static int winBorder = SystemInformation.Border3DSize.Width;

        /// scClockをアクティブウィンドウへ移動する
        private void MoveDisplayTo(RECT rect)
        {
            // 表示位置(X, Y)の取得
            int X = rect.right - winBorder - (buttonW * 4) - this.Width;
            int Y = rect.top + buttonH - this.Height;
            
            // 幅が小さすぎる時はウィンドウの上に移動する
            if (X < rect.left + buttonW)
            {
                X = rect.left + buttonW;
                Y = rect.top - this.Height;
            }
            // 時計を移動
            this.Left = X;
            this.Top = Y;
        }
    }
}
